div .toctree-wrapper > ul {
    column-count: 2;
    margin: 0;
}

ul .toctree-l1 {
    margin: 0;
    -webkit-column-break-inside: avoid;
    page-break-inside: avoid;
    break-inside: avoid-column;
}

div .contents > ul {
    column-count: 2;
    margin: 0;
}

div .contents li {
    margin: 0;
    -webkit-column-break-inside: avoid;
    page-break-inside: avoid;
    break-inside: avoid-column;
}