{% extends "base.html" %}

{% set active_page = page %}

{% block title %} Error {% if error_description %}:{% endif %} {{ error_description }} {% endblock %}

{% block divs %}

    <div class="container-fluid">

        <h1>Sorry, something went wrong :( </h1>

        <p>{{ error_description }} The error message says:</p>
        <p class="error">{{ error_class }} {% if error_class %}:{% endif %} "{{ error_message }}"</p>
        <p>You might want to try to <a onclick="window.history.back()">return and select something else</a> or <a href="/?clear-session=1">clear your session</a> and try again.</p>

    </div>

{% endblock %}