{% extends "base.html" %}

{% set active_page = "logged-in-user" %}

{% block title %} Forgot password {% endblock %}

{% from "security/_macros.html" import render_field_with_errors, render_field %}

{% block divs %}

    <div class="container-fluid">
        <div class="row">
            <div class="col-sm-12">
                {% include "security/_messages.html" %}

                <h1>{{ _('Send password reset instructions') }}</h1>
                <form action="{{ url_for_security('forgot_password') }}" method="POST" name="forgot_password_form">
                    {{ forgot_password_form.hidden_tag() }}
                    {{ render_field_with_errors(forgot_password_form.email) }}
                    {{ render_field(forgot_password_form.submit) }}
                </form>
            </div>
        </div>
    </div>

{% endblock %}