{% extends "base.html" %}

{% set active_page = "logged-in-user" %}

{% block title %} Reset password {% endblock %}


{% from "security/_macros.html" import render_field_with_errors, render_field %}


{% block divs %}

  <div class="container-fluid">
        <div class="row">
            <div class="col-sm-12">
                {% include "security/_messages.html" %}

                <h1>{{ _('Reset password') }}</h1>
                <form action="{{ url_for_security('reset_password', token=reset_password_token) }}" method="POST" name="reset_password_form">
                    {{ reset_password_form.hidden_tag() }}
                    {{ render_field_with_errors(reset_password_form.password) }}
                    {{ render_field_with_errors(reset_password_form.password_confirm) }}
                    {{ render_field(reset_password_form.submit) }}
                </form>
            </div>
        </div>
  </div>

{% endblock%}