"""unique usernames

Revision ID: 1bcccdf0c3e1
Revises: 8fcc5fec67bc
Create Date: 2018-08-08 13:39:56.732107

"""
from alembic import op


# revision identifiers, used by Alembic.
revision = "1bcccdf0c3e1"
down_revision = "8fcc5fec67bc"
branch_labels = None
depends_on = None


def upgrade():
    # ### commands auto generated by Alembic - please adjust! ###
    op.create_unique_constraint(None, "bvp_users", ["username"])
    # ### end Alembic commands ###


def downgrade():
    # ### commands auto generated by Alembic - please adjust! ###
    op.drop_constraint(None, "bvp_users", type_="unique")
    # ### end Alembic commands ###
