"""unique user IDs in data sources

Revision ID: 1a4f0e5c4b86
Revises: 26373d8266db
Create Date: 2018-08-16 12:47:38.492823

"""
from alembic import op


# revision identifiers, used by Alembic.
revision = "1a4f0e5c4b86"
down_revision = "26373d8266db"
branch_labels = None
depends_on = None


def upgrade():
    # ### commands auto generated by Alembic - please adjust! ###
    op.create_unique_constraint(None, "data_sources", ["user_id"])
    # ### end Alembic commands ###


def downgrade():
    # ### commands auto generated by Alembic - please adjust! ###
    op.drop_constraint(None, "data_sources", type_="unique")
    # ### end Alembic commands ###
